﻿Imports System
Imports System.Windows.Forms
Imports NXOpen
Imports NXOpen.UF
Imports NXOpen.Features

Public Class SwitchPlane
   Inherits System.Windows.Forms.Form

   Dim sess As Session = Session.GetSession
   Dim ufs As UFSession = UFSession.GetUFSession
   Dim wp As Part = sess.Parts.Work
   Dim lw As ListingWindow = sess.ListingWindow

   Dim rpls() As DatumPlane = {CType(wp.Datums.FindObject("FIXED_DATUM_PLANE(0)"), DatumPlane), _
                             CType(wp.Datums.FindObject("FIXED_DATUM_PLANE(1)"), DatumPlane), _
                             CType(wp.Datums.FindObject("FIXED_DATUM_PLANE(2)"), DatumPlane)}
   Dim mpl As DatumPlane = CType(wp.Datums.FindObject("FIXED_DATUM_PLANE(3)"), DatumPlane)

   Dim udmname As String = "Switch Plane"
   Dim udm As Session.UndoMarkId = sess.SetUndoMark(Session.MarkVisibility.Visible, udmname)

   Public Shared Sub Main()
      Application.Run(New SwitchPlane)
   End Sub

   Private Sub New()
      MyBase.new()
      Me.InitializeComponent()
   End Sub

   Private Sub btn_ok_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_ok.Click
      Me.Close()
   End Sub

   Private Sub opt_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles opt_1.CheckedChanged, opt_2.CheckedChanged, opt_3.CheckedChanged
      If Not Me.CanFocus Then Exit Sub
      Dim opt As RadioButton = CType(sender, RadioButton)
      If opt.Checked Then
         Dim nr As Integer = CInt(CStr(opt.Name(opt.Name.Length - 1))) - 1
         ChangePlane(rpls(nr))
      End If
   End Sub

   Private Sub chk_orient_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk_orient.CheckedChanged
      If opt_1.Checked Then
         ChangePlane(rpls(0))
      ElseIf opt_2.Checked Then
         ChangePlane(rpls(1))
      ElseIf opt_3.Checked Then
         ChangePlane(rpls(2))
      End If
   End Sub

   Private Sub ChangePlane(ByVal rpl As DatumPlane)
      Dim rplb As DatumPlaneBuilder = wp.Features.CreateDatumPlaneBuilder(rpl.Feature)
      Dim mplb As DatumPlaneBuilder = wp.Features.CreateDatumPlaneBuilder(mpl.Feature)
      Dim dpl As Plane = mplb.GetPlane
      Dim cp(3) As Point3d

      If chk_orient.Checked Then
         rpl.GetCornerPoints(cp(0), cp(1), cp(2), cp(3))
         Dim sc As Double = 0.8

         dpl.Origin = rplb.GetPlane.Origin
         dpl.Matrix = rplb.GetPlane.Matrix

         For i As Integer = 0 To cp.Length - 1
            cp(i).X = rpl.Origin.X + sc * (rpl.Origin.X - cp(i).X)
            cp(i).Y = rpl.Origin.Y + sc * (rpl.Origin.Y - cp(i).Y)
            cp(i).Z = rpl.Origin.Z + sc * (rpl.Origin.Z - cp(i).Z)
         Next
      Else
         mpl.GetCornerPoints(cp(0), cp(1), cp(2), cp(3))
         Dim b As Double = 0.5 * Math.Sqrt((cp(0).X - cp(1).X) ^ 2 + (cp(0).Y - cp(1).Y) ^ 2 + (cp(0).Z - cp(1).Z) ^ 2)
         Dim h As Double = 0.5 * Math.Sqrt((cp(0).X - cp(3).X) ^ 2 + (cp(0).Y - cp(3).Y) ^ 2 + (cp(0).Z - cp(3).Z) ^ 2)
         Dim dlt(,) As Double = {{b, h}, {-b, h}, {-b, -h}, {b, -h}}

         dpl.SetMethod(PlaneTypes.MethodType.Distance)
         dpl.SetGeometry(New NXObject() {rpl})
         dpl.Evaluate()
         dpl.SetMethod(PlaneTypes.MethodType.Fixed)

         For i As Integer = 0 To cp.Length - 1
            cp(i).X = dpl.Origin.X + dlt(i, 0) * dpl.Matrix.Xx + dlt(i, 1) * dpl.Matrix.Yx
            cp(i).Y = dpl.Origin.Y + dlt(i, 0) * dpl.Matrix.Xy + dlt(i, 1) * dpl.Matrix.Yy
            cp(i).Z = dpl.Origin.Z + dlt(i, 0) * dpl.Matrix.Xz + dlt(i, 1) * dpl.Matrix.Yz
         Next
      End If

      Try
         mplb.SetCornerPoints(cp(0), cp(1), cp(2), cp(3))
      Catch ex As Exception
         lw.Open()
         lw.WriteLine(ex.Message)
      End Try

      mpl.SetNameLocation(cp(0))
      mplb.Commit()
      mplb.Destroy()
      rplb.Destroy()

      sess.UpdateManager.DoUpdate(udm)
   End Sub

#Region "VS Designer Code"

   'Das Formular überschreibt den Löschvorgang, um die Komponentenliste zu bereinigen.
   <System.Diagnostics.DebuggerNonUserCode()> _
   Protected Overrides Sub Dispose(ByVal disposing As Boolean)
      Try
         If disposing AndAlso components IsNot Nothing Then
            components.Dispose()
         End If
      Finally
         MyBase.Dispose(disposing)
      End Try
   End Sub

   'Wird vom Windows Form-Designer benötigt.
   Private components As System.ComponentModel.IContainer

   'Hinweis: Die folgende Prozedur ist für den Windows Form-Designer erforderlich.
   'Das Bearbeiten ist mit dem Windows Form-Designer möglich.  
   'Das Bearbeiten mit dem Code-Editor ist nicht möglich.
   <System.Diagnostics.DebuggerStepThrough()> _
   Private Sub InitializeComponent()
      Me.opt_1 = New System.Windows.Forms.RadioButton
      Me.opt_2 = New System.Windows.Forms.RadioButton
      Me.opt_3 = New System.Windows.Forms.RadioButton
      Me.btn_ok = New System.Windows.Forms.Button
      Me.chk_orient = New System.Windows.Forms.CheckBox
      Me.SuspendLayout()
      '
      'opt_1
      '
      Me.opt_1.AutoSize = True
      Me.opt_1.Location = New System.Drawing.Point(22, 11)
      Me.opt_1.Name = "opt_1"
      Me.opt_1.Size = New System.Drawing.Size(61, 17)
      Me.opt_1.TabIndex = 0
      Me.opt_1.Text = "Plane 1"
      Me.opt_1.UseVisualStyleBackColor = True
      '
      'opt_2
      '
      Me.opt_2.AutoSize = True
      Me.opt_2.Location = New System.Drawing.Point(22, 34)
      Me.opt_2.Name = "opt_2"
      Me.opt_2.Size = New System.Drawing.Size(61, 17)
      Me.opt_2.TabIndex = 1
      Me.opt_2.Text = "Plane 2"
      Me.opt_2.UseVisualStyleBackColor = True
      '
      'opt_3
      '
      Me.opt_3.AutoSize = True
      Me.opt_3.Location = New System.Drawing.Point(22, 57)
      Me.opt_3.Name = "opt_3"
      Me.opt_3.Size = New System.Drawing.Size(61, 17)
      Me.opt_3.TabIndex = 2
      Me.opt_3.Text = "Plane 3"
      Me.opt_3.UseVisualStyleBackColor = True
      '
      'btn_ok
      '
      Me.btn_ok.Location = New System.Drawing.Point(15, 125)
      Me.btn_ok.Name = "btn_ok"
      Me.btn_ok.Size = New System.Drawing.Size(75, 23)
      Me.btn_ok.TabIndex = 3
      Me.btn_ok.Text = "OK"
      Me.btn_ok.UseVisualStyleBackColor = True
      '
      'chk_orient
      '
      Me.chk_orient.AutoSize = True
      Me.chk_orient.Location = New System.Drawing.Point(13, 86)
      Me.chk_orient.Name = "chk_orient"
      Me.chk_orient.Size = New System.Drawing.Size(79, 30)
      Me.chk_orient.TabIndex = 4
      Me.chk_orient.Text = "Use Target" & Global.Microsoft.VisualBasic.ChrW(13) & Global.Microsoft.VisualBasic.ChrW(10) & "Orientation"
      Me.chk_orient.UseVisualStyleBackColor = True
      '
      'Form1
      '
      Me.AutoScaleDimensions = New System.Drawing.SizeF(6.0!, 13.0!)
      Me.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font
      Me.ClientSize = New System.Drawing.Size(104, 153)
      Me.ControlBox = False
      Me.Controls.Add(Me.chk_orient)
      Me.Controls.Add(Me.btn_ok)
      Me.Controls.Add(Me.opt_3)
      Me.Controls.Add(Me.opt_2)
      Me.Controls.Add(Me.opt_1)
      Me.MaximumSize = New System.Drawing.Size(112, 180)
      Me.MinimumSize = New System.Drawing.Size(112, 180)
      Me.Name = "Form1"
      Me.ShowIcon = False
      Me.ShowInTaskbar = False
      Me.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide
      Me.Text = "   Select a Plane"
      Me.TopMost = True
      Me.ResumeLayout(False)
      Me.PerformLayout()

   End Sub

   Friend WithEvents opt_1 As System.Windows.Forms.RadioButton
   Friend WithEvents opt_2 As System.Windows.Forms.RadioButton
   Friend WithEvents opt_3 As System.Windows.Forms.RadioButton
   Friend WithEvents btn_ok As System.Windows.Forms.Button
   Friend WithEvents chk_orient As System.Windows.Forms.CheckBox

#End Region

End Class
